# Copyright (c) 1995 Merck and Co., Inc.
#       Whitehouse Station, NJ USA
#          All Rights Reserved

proc Nv_MkPeakPick {nm} {
    
    set varnm [NvVarnm $nm] ; #remove '.' from $nm, put in $varnm
    
    if {[string range $nm 0 0]!="."} {
        errmsg "Window must start with ."
        return
    }
    
    global $varnm filtnm ; # alias $filtnm to $varnm
    
    set filtnm(list) ""
    upvar #0 $varnm panvar ; # alias $panvar to $varnm
    
    if {[winfo exists $nm]} {
        destroy $nm
        unset $varnm
    }
    
    toplevel $nm
    wm withdraw $nm
    
    set ndim 4
    
    frame $nm.fr1 -bd 2 ; # set up all the frames in the peak picking window
    frame $nm.fr2 -bd 2
    frame $nm.fr3 -bd 2
    frame $nm.fr4a
    frame $nm.fr4
    frame $nm.fr5
    frame $nm.fr6
    frame $nm.fr7
    frame $nm.fr8 -bd 2 -relief sunken
    pack $nm.fr1 $nm.fr2  $nm.fr3 $nm.fr4a $nm.fr4 $nm.fr5 $nm.fr6 $nm.fr7 $nm.fr8 -side top -fill x -expand yes
    
    label $nm.fr1.lbl -text "List Name:" -width 12
    entry $nm.fr1.list -width 16 -relief sunken -textvariable ${varnm}(list)
    set panvar(list) ""
    menubutton $nm.fr1.select -text Select -underline 0 \
    -menu $nm.fr1.select.menu -width 7 -relief raised
    
    pack $nm.fr1.lbl $nm.fr1.list  $nm.fr1.select  -side left
    
    set menu $nm.fr1.select.menu
    menu $menu -postcommand "NvFillPeakMenu $menu ${varnm}(list) {}  0"
    
    
    
    label $nm.fr2.label -text "Pick Region:" -width 12
    pack $nm.fr2.label -side left
    
    foreach button "Window Box Filter" i "0 1 4" {
        set bn [string tolower $button]
        radiobutton $nm.fr2.$bn -text $button -variable ${varnm}(region)\
        -value $i -command "getFiltInf $i $nm"
        pack $nm.fr2.$bn -side left
    }
    set panvar(region) 0
    
    label $nm.fr3.label -text "Pick Mode:" -width 12
    pack $nm.fr3.label -side left
    set i 0
    
    foreach button "New Replace Append" {
        set bn [string tolower $button]
        radiobutton $nm.fr3.$bn -text $button -variable ${varnm}(mode)\
        -value $i
        pack $nm.fr3.$bn -side left
        incr i
    }
    set panvar(mode) 0
    
    label $nm.fr4a.label -text "Dimension:" -width 12 -anchor w
    pack $nm.fr4a.label -side left
    for {set i 1} {$i<=$ndim} {incr i}  {
        label $nm.fr4a.lbl$i -text $i -width 6
        pack $nm.fr4a.lbl$i -side left
    }
    label $nm.fr4.label -text "Wrap:" -width 12 -anchor w
    pack $nm.fr4.label -side left
    set i 0
    
    for {set i 1} {$i<=$ndim} {incr i}  {
        set bn fld$i
        checkbutton $nm.fr4.$bn -variable ${varnm}(fld,$i) -width  4
        set panvar(fld$i) 0
        pack $nm.fr4.$bn -side left
    }
    
    label $nm.fr5.label -text "Min:" -width 12 -anchor w
    pack $nm.fr5.label -side left
    set i 0
    
    for {set i 1} {$i<=$ndim} {incr i}  {
        set bn minw$i
        entry $nm.fr5.$bn  -textvariable ${varnm}(minw,$i) -width 6
        set panvar(minw,$i) 3.0
        pack $nm.fr5.$bn -side left
    }
    
    label $nm.fr6.label -text "Max:" -width 12 -anchor w
    pack $nm.fr6.label -side left
    set i 0
    
    for {set i 1} {$i<=$ndim} {incr i}  {
        set bn maxw$i
        entry $nm.fr6.$bn  -textvariable ${varnm}(maxw,$i) -width 6
        set panvar(maxw,$i) 100.0
        pack $nm.fr6.$bn -side left
    }
    
    label $nm.fr7.label -text "Thickness:" -width 12 -anchor w
    pack $nm.fr7.label -side left
    entry $nm.fr7.thick  -textvariable ${varnm}(thickness) -width 6
    pack $nm.fr7.thick
    set panvar(thickness) 0
    
    
    button $nm.fr8.pick -text Pick -command "NvDoPeakPick $nm"
    button $nm.fr8.close -text Close -command "wm withdraw $nm"
    pack $nm.fr8.pick $nm.fr8.close -side left
    
    
    
}
proc getFiltInf {region nm} {
    
    destroy $nm.fr9
    destroy $nm.fr10
    destroy $nm.fr11
    
    if {$region == 4} {
        frame $nm.fr9
        frame $nm.fr10
        frame $nm.fr11
        pack $nm.fr9 $nm.fr10 $nm.fr11 -after $nm.fr2  -side top -fill x -expand yes
        
        label $nm.fr9.lbl -text "Filter List:" -width 12
        entry $nm.fr9.list -width 16 -relief sunken -textvariable filtnm(list)
        menubutton $nm.fr9.select -text Select -underline 0 \
        -menu $nm.fr9.select.menu -width 12 -relief raised
        
        
        pack $nm.fr9.lbl $nm.fr9.list  $nm.fr9.select  -side left
        
        set menu $nm.fr9.select.menu
        menu $menu -postcommand "NvFillPeakMenu $menu filtnm(list) \"MkFiltPan $nm\"  0"
    }
    
}

proc MkFiltPan {nm} {
    
    global filtnm
    set varnm [NvVarnm $nm]
    upvar #0 $varnm panvar
    set ndim 4
    
    foreach child [winfo child $nm.fr10] {
        destroy $child
    }
    foreach child [winfo child $nm.fr11] {
        destroy $child
    }
    
    if { $filtnm(list) != "none"} {
        label $nm.fr10.label -text "Filter Axes:" -width 12 -anchor w
        pack $nm.fr10.label -side left
        set i 0
        
        
        foreach dim [nv_peak label $filtnm(list)] {
            set bn fLbl$dim
            set panvar(axisLbl,$dim) $dim
            label $nm.fr10.$bn  -textvariable [set varnm](axisLbl,$dim) -width 7
            pack $nm.fr10.$bn -side left
        }
        set ndim [llength  [eval nv_peak label $filtnm(list) ]]
        
        
        label $nm.fr11.label -text "Correlate Axes:" -width 12 -anchor w
        pack $nm.fr11.label -side left
        
        set dataSet $panvar(dataset)
        set dataNdim [eval nv_dataset ndim $dataSet ]
        set dataLabels "None"
        for {set j 1} {$j<=$dataNdim} {incr j} {
            lappend dataLabels  [eval nv_dataset label $dataSet $j ]
            
        }
        
        
        foreach dim [nv_peak label $filtnm(list)] {
            set bn d$dim
            set caxis $dim
            eval tk_optionMenu $nm.fr11.$bn filtnm(caxis,$dim)  $dataLabels
            $nm.fr11.$bn config -width 6
            
            pack $nm.fr11.$bn -side left
        }
    }
    
}


proc NvDoPeakPick {nm} {
    global filtnm
    set varnm [NvVarnm $nm]
    upvar #0 $varnm panvar
    
    if {$panvar(region)== 0 || $panvar(region)==1} {
        nv_peak pick -var $varnm -dataset $panvar(dataset) $panvar(list)
        
    } elseif { $panvar(region)== 4 } {
        
        set caxis ""
        set fLbl ""
        foreach dim [nv_peak label $filtnm(list)] {
            lappend caxis $filtnm(caxis,$dim)
            lappend fLbl $dim
        }
        
        set nDim [eval nv_dataset ndim $panvar(dataset)]
        for {set i 1} {$i <= $nDim} {incr i} {
            set nPts [eval nv_dataset size $panvar(dataset) $i]
            eval nv_win pt $i 1 $nPts
        }
        
        set dataPpm [eval nv_win ppm]
        set dataPpmAll [eval nv_win ppm]
        set lvl [nv_win lvl]
        
        foreachpeak peak $filtnm(list) {
            
            for {set i 0} {$i < [llength $fLbl] } {incr i} {
                set caxisLbl [lindex $caxis $i]
                set faxisLbl [lindex $fLbl $i]
                
                if {$caxisLbl != "None"} {
                    set cppm [eval nv_peak elem $faxisLbl.P $filtnm(list).$peak]
                    set cwidth [eval nv_peak elem $faxisLbl.W $filtnm(list).$peak]
                    set j 1
                    while { [nv_dataset label $panvar(dataset) $j] != "$caxisLbl"} {
                        incr j
                    }
                    set j [expr $j-1]
                    
                    set dataPpm [lreplace $dataPpm $j $j "[eval expr $cppm - $cwidth] [eval expr $cppm + $cwidth]"]
                }
            }
            
            switch $nDim {
                1 {set d1 [lsort -decreasing [lindex $dataPpm 0] ]
                eval nv_peak pick -var $varnm -lvl $lvl -filter -dataset $panvar(dataset) -d1 $d1 $panvar(list)}
                2 {set d1 [lsort -decreasing [lindex $dataPpm 0] ]
                    set d2 [lsort -decreasing [lindex $dataPpm 1] ]
                eval nv_peak pick -var $varnm -lvl $lvl -filter -dataset $panvar(dataset) -d1 $d1 -d2 $d2 $panvar(list)}
                3 {set dm1 [lsort -decreasing [lindex $dataPpm 0] ]
                    set dm2 [lsort -decreasing [lindex $dataPpm 1] ]
                    set dm3 [lsort -decreasing [lindex $dataPpm 2] ]
                eval nv_peak pick -var $varnm -lvl $lvl -filter -dataset $panvar(dataset) -d1 $dm1 -d2 $dm2 -d3 $dm3 $panvar(list)}
                4 {set d1 [lsort -decreasing [lindex $dataPpm 0] ]
                    set d2 [lsort -decreasing [lindex $dataPpm 1] ]
                    set d3 [lsort -decreasing [lindex $dataPpm 2] ]
                    set d4 [lsort -decreasing [lindex $dataPpm 3] ]
                eval nv_peak pick -var $varnm -lvl $lvl -filter -dataset $panvar(dataset) -d1 $d1 -d2 $d2 -d3 $d3 -d4 $d4 $panvar(list)}
            }
            set panvar(mode) 2
        }
        
        
        set dataSet $panvar(dataset)
        set dataNdim [eval nv_dataset ndim $dataSet ]
        set dataLabels ""
        for {set j 1} {$j<=$dataNdim} {incr j} {
            set label [eval nv_dataset label $dataSet $j ]
            set width($label) 0
            lappend dataLabels  [eval nv_dataset label $dataSet $j ]
        }
        
        set count 0
        set template ""
        foreachpeak peak $panvar(list) {
            set status [eval nv_peak status $peak]
            if { $status == 0} {
                foreach el $dataLabels {
                    set tmp [eval nv_peak elem $el.W $panvar(list)]
                    set width($el) [expr $tmp + $width($el)]
                }
                set count [expr $count+1]
            }
        }
        
        foreach el  $dataLabels {
            set width($el) [expr $width($el)/$count*0.4]
            lappend template $el
            lappend template $width($el)
        }
        eval nv_peak template $panvar(list) $template
        nv_peak cluster $panvar(list)
        NvPeakDelOverlaps $panvar(list)
        
        
    }
    $panvar(window) dispeaks $panvar(list)
    NvStarAddPeakList $panvar(list)
}

proc NvPeakDelOverlaps {listName} {
    foreachpeak peak $listName {
        set status [eval nv_peak status $peak]
        if {$status != -1} {
            set links [eval nv_peak links $peak]
            set max 0.0
            if {[llength $links] != 1} {
                foreach lpeak $links {
                    set int [eval nv_peak elem int $lpeak]
                    if {[expr {abs($int) > $max}]} {
                        set max [expr {abs($int)}]
                        set maxpeak $lpeak
                    }
                }
                foreach lpeak $links {
                    if {$lpeak != $maxpeak} {
                        eval nv_peak status $lpeak -1
                    }
                }
            }
        }
    }
}



proc NvShowPickPanel args {
    global Nv_Priv
    if {[llength $args]>0} { set nm [lindex $args 0] } else {set nm $Nv_Priv(peakPick)}
    if {![winfo exists $nm]} {
        Nv_MkPeakPick $nm
    }
    wm deiconify $nm
    raise $nm
    set varnm [NvVarnm $nm]
    upvar #0 $varnm panvar
    set panvar(dataset) [nv_win dataset]
    set panvar(window) [nv_win active]
    if {[nv_win poscolor] == "None"} {
        set panvar(pos) 0
    } else {
        set panvar(pos) 1
    }
    if {[nv_win negcolor] == "None"} {
        set panvar(neg) 0
    } else {
        set panvar(neg) 1
    }
    wm title $nm $panvar(dataset)
}

proc NvAddPeak {win x y rmode} {
    set varnm [NvVarnm .peakPick]
    
    global $varnm
    upvar #0 $varnm panvar
    set ndim 4
    
    set panvar(list) [$win dispeaks]
    if {($panvar(list) == "") || ($panvar(list) == "none")} {
        set panvar(list) [file root [file tail [$win dataset]]]
        if {[lsearch [nv_peak list] $panvar(list)] != -1} {
            set panvar(mode) 2
        } else {
            set panvar(mode) 0
        }
        
    } else {
        set panvar(mode) 2
    }
    set panvar(dataset) [$win dataset]
    set panvar(region) [expr 2+$rmode]
    #   set panvar(thickness) 0
    set xy [win pix2ppm $x $y]
    set panvar(p,1) [lindex $xy 0]
    set panvar(p,2) [lindex $xy 1]
    $win pick -var $varnm -dataset $panvar(dataset) $panvar(list)
}

proc NvDelPeak {win x y} {
    set xy [win pix2ppm $x $y]
    set p1 [lindex $xy 0]
    set p2 [lindex $xy 1]
    set peak [nv_peak closest [win dispeak] $p1 $p2 ]
    nv_peak delete $peak
}


proc NvPeakBusy {mode {display ""}} {
    global Nv_Priv int_flag
    switch $mode {
        on {
            set Nv_Priv(pkbusyAfter) [after 2000 NvPeakBusy show]
            set Nv_Priv(pkbusyActive) 0
            # update
        }
        off {
            catch "after cancel $Nv_Priv(pkbusyAfter)"
            if {[winfo exists $Nv_Priv(pkbusyWin)]} {
                destroy $Nv_Priv(pkbusyWin)
            }
            set Nv_Priv(pkbusyActive) 0
            
        }
        show {
            if {[info exists Nv_Priv(pkbusyWin)]} {
                destroy $Nv_Priv(pkbusyWin)
            }
            set win [nv_win active]
            set tlwin [winfo toplevel $win]
            set x [winfo x $tlwin]
            set y [winfo y $tlwin]
            set tl .cancelWin
            toplevel $tl -width 100 -height 30
            wm geometry $tl +$x+$y
            grab $tl
            set Nv_Priv(pkbusyWin) $tl
            button $tl.b -text Cancel -command {set int_flag 1;destroy $Nv_Priv(pkbusyWin)}
            label $tl.l -textvariable Nv_Priv(pkbusyCounter) -text "" -width 6
            pack $tl.b $tl.l -side left
            raise $tl
            set Nv_Priv(pkbusyActive) 1
        }
        check {
            #update
            if {$Nv_Priv(pkbusyActive)} {
                set Nv_Priv(pkbusyCounter)  $display
                if { $int_flag} {
                    set int_flag 0
                    return 1
                } else {
                    raise $Nv_Priv(pkbusyWin)
             #       update
                    return 0
                }
            } else {
                set int_flag 0
                return 0
            }
        }
    }
    
    
}
