# script history: part of Merck distribution
# edited 9/99 c hastings to add a few functions

# Copyright (c) 1995 Merck and Co., Inc.
#       Whitehouse Station, NJ USA
#          All Rights Reserved

proc NvVarnm {name} {
    set varnm Nv[string range $name 1 end]; #remove first char from the string "name" 
    return $varnm
}
proc Nv_HelpMessage args {
    global Nv_CmdHelp
    if {[llength $args]==1} {
        set cmd [lindex $args 0]
        NvHelpCheck $cmd
        if {[info exists Nv_CmdHelp($cmd)]} {
            puts $Nv_CmdHelp($cmd)
            return
        } else {return "No help for $cmd"}
    } elseif {[llength $args]==2} {
        set cmd [lindex $args 0]
        set option [lindex $args 1]
        NvHelpCheck $cmd
        if {[info exists Nv_CmdHelp($cmd,$option)]} {
            puts $Nv_CmdHelp($cmd,$option)
            return
        } else {return "No help for $cmd $option"}
    } else { error "Nv_HelpMessage: should be one or two arguments"}
}

proc NvHelpCheck {cmd} {
    global NVH Nv_CmdHelp
    set filename $NVH/help/$cmd.help
    if {![info exists Nv_CmdHelp($cmd)]} {
        if {[file exists $filename]} {
            NvParseHelp $filename
        } else {
            error "File $filename doesn't exist"
        }
    }
    if {![info exists Nv_CmdHelp($cmd)]} {
        error "Nv_CmdHelp($cmd) not in help file $filename"
    }
    return
}
proc NvFillPeakMenu {menu var command useI} {
    $menu delete 0 end
    if {$useI} {
        $menu add radiobutton -label none -variable $var -value -1 -command $command
    } else {
        $menu add radiobutton -label none -variable $var -value none -command $command
    }
    set iValue 0
    foreach list [nv_peak list] {
        if {$useI} {
            set val $iValue
            incr iValue
        } else {
            set val $list
        }
        $menu add radiobutton -label $list -variable $var -value $val -command $command
    }
}

proc NvFillDatasetsMenu {menu var command useI} {
    $menu delete 0 end
    set iValue 0
    foreach dataset [nv_dataset open] {
        if {$useI} {
            set val $iValue
            incr iValue
        } else {
            set val $dataset
        }
        $menu add radiobutton -label $dataset -variable $var -value $val -command $command
    }
}

proc NvAddWidget {nm w gvar args} {
    upvar #0 gvar gv
    if {[llength $args] > 0} {
        set addWidgMode [lindex $args 0]
    }
    if {![info exists gv(iFrame)]} {
        set gv(iFrame) 0
    }
    if {![info exists gv(iItem)]} {
        set gv(iItem) 0
    }
    set type [lindex $w 0]
    switch $type {
        oframe {
            set item [lindex $w 1]
            set gv(oframe) $nm.$item
            frame $gv(oframe) -border 5 -relief raised
            pack $gv(oframe) -side top -fill x -anchor n
        }
        frame {
            set gv(frame) $gv(oframe).fr$gv(iFrame)
            frame $gv(frame)
            pack $gv(frame) -side top -fill x -expand y
            incr gv(iFrame)
            set gv(iItem) 0
        }
        entry {
            set item [lindex $w 1]
            set label [lindex $w 2]
            set var [subst [lindex $w 3]]
            label $gv(frame).label$item -text $label
            pack $gv(frame).label$item -side left
            entry $gv(frame).entry$item -width 6 -textvariable $var
            pack $gv(frame).entry$item -side left
            if {[llength $w]>6} {
                eval $gv(frame).entry$item config [lindex $w 6]
            }
            
        }
        label {
            set item [lindex $w 1]
            label $gv(frame).label$item -text $item
            if {[llength $w]>2} {
                eval $gv(frame).label$item config [lindex $w 2]
            }
            pack $gv(frame).label$item -side left -expand yes
            
        }
        button {
            set item [lindex $w 1]
            set action [lindex $w 2]
            if {[llength $action]>2} {
                set action [lreplace $action 2 2 $gv(frame).button$item]
            }
            button $gv(frame).button$item -text $item -command $action
            pack $gv(frame).button$item -side left -expand yes
        }
        scale {
            set var [subst [lindex $w 1]]
            set values [lindex $w 2]
            scale $gv(frame).scale$gv(iItem) -resolution [lindex $values 3] -from [lindex $values 1] -to [lindex $values 2] -variable $var -orient horizontal
            pack $gv(frame).scale$gv(iItem) -side left
        }
        escale {
            set var [subst [lindex $w 1]]
            set values [lindex $w 2]
            entry $gv(frame).entry$gv(iItem) -width 4 -textvariable $var
            scale $gv(frame).scale$gv(iItem) -resolution [lindex $values 3] -from [lindex $values 1] -to [lindex $values 2]  -variable $var -orient horizontal -showvalue no
            pack $gv(frame).entry$gv(iItem) -side left
            pack $gv(frame).scale$gv(iItem) -side left
        }
        radiobutton {
            set var [subst [lindex $w 1]]
            foreach item [lindex $w 2] {
                radiobutton $gv(frame).button$item -text $item -variable  $var -value $item
                pack $gv(frame).button$item -side left -expand yes
            }
            set $var [lindex $w 2]
        }
        checkbutton {
            set item [lindex $w 1]
            set var [subst [lindex $w 2]]
            checkbutton $gv(frame).cb$item -text $item -variable  $var
            pack $gv(frame).cb$item -side left -expand yes
            set $var [lindex $w 3]
        }
    }
    incr gv(iItem)
}
catch "rename exit exit.old"


proc NvCenterWin {tl {x1 0.5} {y1 0.45} } {
    if {![winfo exists $tl]} {
        return
    }
    set width [winfo reqwidth $tl]
    set height [winfo reqheight $tl]
    
    set x [expr round([winfo screenwidth $tl]*$x1-$width/2)]
    set y [expr round([winfo screenheight .]*$y1-$height/2)]
    
    wm geometry $tl +$x+$y
}


proc nvsrc {scriptName} {
    global NVH
    source [file join $NVH nvtcl $scriptName.tcl]
}

#source ::library:http2.0:http.tcl
#package require http 2.0
proc GetBMRB {file} {
    set url http://www.bmrb.wisc.edu/data_library/files/bmr$file.str
    set f1 [open bmr$file.str w]
    set token [http::geturl $url -channel $f1]
    puts $token
    upvar #0 $token state
    set lines [split $state(body) \n]
    close $f1
    puts $state(status)
    if {$state(status) != "ok"} {
        puts "GetBMRB failed with status $state(status)"
        file delete bmr$file.str
    } else {
        StarReset
        Star bmr$file.str
    }
    
    return
}

proc exit {{status 0}} {
    global Nv_Prefs NvStar Nv_Priv
    
    if {([info exists NvStar(dirty)]) && $NvStar(dirty)} {
        set result [tk_messageBox -icon question -type yesnocancel -message "Save database changes before exiting?"]
        if {$result == "cancel"} {return}
        if {$result == "no"} {nv_exit $status}
        if {$result == "yes"} {
            
            set types [format {
                {{%s} {%s} }
            } $Nv_Priv(filebox,starWrite,description) $Nv_Priv(filebox,starWrite,filter)]
            
            if {$Nv_Prefs(usedirs)} {
                set idir $Nv_Prefs(dir,starRead)
            } else {
                set idir [pwd]
            }
            set fileName [tk_getSaveFile -filetypes $types -title $Nv_Priv(filebox,starWrite,title) -initialdir $idir]
            if {$fileName != ""} {
                StarWrite $filename
            } else {
                return
            }
        }
    }
    
    if {(![info exists Nv_Prefs(override,exit)]) || $Nv_Prefs(override,exit)} {
        nv_exit $status
    }
    
    wm deiconify .
    raise .
    set result [tk_messageBox -icon question -type yesno -message "Leave the program?"]
    if {$result=="no"} {
        return
    }
    nv_exit $status
}
