# script history: part of Merck distribution
# modified version for use with overlay2d: K Gardner.

# Copyright (c) 1995 Merck and Co., Inc.
#       Whitehouse Station, NJ USA
#          All Rights Reserved

proc NvMkSpecAttr {nm} {
    if {[string range $nm 0 0]!="."} {
        error "Window must start with ."
        return
    }
    set varnm [NvVarnm $nm]
    upvar #0 $varnm panvar
    
    global $varnm Nv_Priv
    
    if {[winfo exists $nm]} {
        destroy $nm
        unset $varnm
    }
    
    toplevel $nm
    wm protocol $nm WM_DELETE_WINDOW "wm withdraw $nm"
    set frame $nm
    frame $frame.mbar -relief raised -bd 2
    frame $frame.sliders
    pack $frame.mbar $frame.sliders -side top -fill x
    
    frame $frame.sliders.left
    frame $frame.sliders.right
    pack $frame.sliders.left $frame.sliders.right -side left -fill both
    set frame $nm.sliders.left
    frame $frame.lvlFrame
    frame $frame.mulFrame
    frame $frame.nlvlFrame
    pack $frame.lvlFrame $frame.mulFrame $frame.nlvlFrame -side top -fill x
    
    set frame $nm.sliders.right
    frame $frame.sclFrame
    frame $frame.xoffFrame
    frame $frame.yoffFrame
    pack $frame.sclFrame $frame.xoffFrame $frame.yoffFrame -side top -fill x
    
    set frame $nm.mbar
    button $frame.draw -text Draw -command NvSpectrumDraw
    add_tip $frame.draw specAttr.draw "Click this button to draw the spectrum"
    
    
    menubutton $frame.file -text File -underline 0 -menu $frame.file.menu
    add_tip $frame.file specAttr.file "File Menu:\n\n Dataset: Open a panel to select dataset for window.\n\n Close: Close Spectrum Attributes Panel."
    
    menubutton $frame.colors -text Colors -underline 0 -menu $frame.colors.menu
    add_tip $frame.colors specAttr.colors "Color Menu:\n\n Positive: Set color for positive contours.\n\n Negative: Set color for negative contours."
    
    menubutton $frame.dims -text Dimensions -underline 0 -menu $frame.dims.menu
    add_tip $frame.dims specAttr.dims "Dimensions Menu:\n\n 1Dx: Draw vectors along parallel to plot x axis.\n\n 1Dy: Draw vectors along parallel to plot y\
    axis.\n\n 2D:  Draw 2D contour plots."
    
    menubutton $frame.mode -text Mode -underline 0 -menu $frame.mode.menu
    add_tip $frame.mode specAttr.mode "Mode Menu:\n\n Record: Record contours to file while drawing.\n\n Play:   Draw contours from prerecorded file.\n\n\
    Live:   Calculate and draw contours from original dataset."
    
    menubutton $frame.misc -text Misc -underline 0 -menu $frame.misc.menu
    add_tip $frame.misc specAttr.misc "Miscellaneous Menu:\n\n AutoClear: Clear spectrum window before drawing next spectrum.\n
    AutoLvl: Set contour lvl at 5x std. dev. of lower left corner.\n
    Overlay2D: Control parameters for overlays of 2D spectra."
    
    pack $frame.file $frame.colors $frame.dims $frame.mode $frame.misc -side left -padx 2 -pady 2
    pack $frame.draw -side right -padx 2 -pady 2
    menu $frame.file.menu -tearoff 0
    $frame.file.menu add command -label Dataset -command "NvShowDatasetList"
    $frame.file.menu add command -label Close -command "wm withdraw $nm"
    menu $frame.colors.menu -tearoff 0
    $frame.colors.menu add cascade -label Positive -menu $frame.colors.menu.pos
    $frame.colors.menu add cascade -label Negative -menu $frame.colors.menu.neg
    menu $frame.colors.menu.pos -tearoff 0
    menu $frame.colors.menu.neg -tearoff 0
    foreach color $Nv_Priv(colorlist) {
        set fcolor $color
        if {$fcolor=="None"} {
            set fcolor white
        }
        $frame.colors.menu.pos add radiobutton -label $color -foreground $fcolor -command "nv_win poscolor $color" -variable ${varnm}(poscolor) -value $color
        $frame.colors.menu.neg add radiobutton -label $color -foreground $fcolor -command "nv_win negcolor $color" -variable ${varnm}(negcolor) -value $color
    }
    menu $frame.dims.menu -tearoff 0
    $frame.dims.menu add radiobutton -label 1Dx -command "nv_win dim 0" -variable ${varnm}(dim) -value 0
    $frame.dims.menu add radiobutton -label 1Dy -command "nv_win dim 1" -variable ${varnm}(dim) -value 1
    $frame.dims.menu add radiobutton -label 2D -command "nv_win dim 2" -variable ${varnm}(dim) -value 2
    menu $frame.mode.menu -tearoff 0
    $frame.mode.menu add radiobutton -label Live -command "nv_win mode live" -variable ${varnm}(mode) -value live
    $frame.mode.menu add radiobutton -label Record -command "nv_win mode record" -variable ${varnm}(mode) -value record
    $frame.mode.menu add radiobutton -label Play -command "nv_win mode play" -variable ${varnm}(mode) -value play
    menu $frame.misc.menu -tearoff 0
    $frame.misc.menu add checkbutton -label AutoClear -variable ${varnm}(aclear)
    $frame.misc.menu add command -label AutoLvl -command NvSpectrumSetLvl
    $frame.misc.menu add command -label Overlay2D -command OverlayAttr
    focus $frame
    
    
    set scaleWidth 2.5c
    set entryWidth  10
    set left $nm.sliders.left
    set frame $left.lvlFrame
    
    label $frame.lvlLabel -text Lvl -width 5
    button $frame.lvlDown -bitmap @$Nv_Priv(image,down)  -width 20 -command "NvChangeLvl down $varnm"
    add_tip $frame.lvlDown specAttr.lvlDown "Click this button to decrease the contour level."
    
    button $frame.lvlUp -bitmap @$Nv_Priv(image,up) -width 20 -command "NvChangeLvl up $varnm"
    add_tip $frame.lvlUp specAttr.lvlUp "Click this button to increase the contour level."
    
    entry $frame.lvlEntry -width $entryWidth -relief sunken -textvariable ${varnm}(lvl)
    add_tip $frame.lvlEntry specAttr.lvlEntry {Enter the contour level here.  The contour \
    level specifies the intensity of the lowest contour that will be drawn.}
    
    pack $frame.lvlLabel $frame.lvlDown $frame.lvlUp  $frame.lvlEntry -side left
    
    
    set entryWidth  7
    set frame $left.mulFrame
    
    label $frame.mulLabel -text CLM -width 5
    scale $frame.mulScale -from 1.0 -to 3.0 -length $scaleWidth -resolution 0.01 -orient horizontal \
    -variable ${varnm}(clm)  -showvalue false -command "NvSetAttrPar $varnm clm"
    
    add_tip $frame.mulScale specAttr.mulScale {Use this scale bar to set the contour level multiplier.}
    
    entry $frame.mulEntry -width $entryWidth -relief sunken -textvariable ${varnm}(clm)
    add_tip $frame.mulEntry specAttr.mulEntry {Enter the contour level multiplier here.  \
    Each contour level will be drawn at an intensity that is this value times the intensity level of the previous contour.}
    pack $frame.mulLabel $frame.mulScale $frame.mulEntry -side left
    
    set frame $left.nlvlFrame
    label $frame.nlvlLabel -text NLvls -width 5
    scale $frame.nlvlScale -from 1 -to 50.0 -length $scaleWidth -resolution 1 -orient horizontal \
    -variable ${varnm}(nlvl)  -showvalue false -command "NvSetAttrPar $varnm nlvl "
    add_tip $frame.nlvlScale specAttr.nlvlScale {Use this scale bar to set the maximum number of contours that will be drawn.}
    
    entry $frame.nlvlEntry -width $entryWidth -relief sunken -textvariable ${varnm}(nlvl)
    add_tip $frame.nlvlEntry specAttr.nlvlEntry "Enter the the number of contours here.  This\
    determines the maximum number of contours that will be drawn."
    
    pack $frame.nlvlLabel $frame.nlvlScale $frame.nlvlEntry -side left
    
    set right $nm.sliders.right
    set frame $right.sclFrame
    
    label $frame.sclLabel -text Scale -width 5
    scale $frame.sclScale -from 0.1 -to 100.0 -length $scaleWidth -resolution 0.1 -orient horizontal \
    -variable ${varnm}(scale) -showvalue false -command "DrawSlice x scale"
    add_tip $frame.sclScale specAttr.sclScale "Use this scale bar to enter the vscale value used when drawing vectors."
    
    entry $frame.sclEntry -width $entryWidth -relief sunken -textvariable ${varnm}(scale)
    bind $frame.sclEntry <Return> "DrawSlice x scale"
    add_tip $frame.sclEntry specAttr.sclEntry "Enter the value of the scale used when drawing vectors.  \
    The larger the number the smaller the drawn vectors will be."
    
    pack $frame.sclLabel $frame.sclScale $frame.sclEntry -side left
    
    set frame $right.xoffFrame
    
    label $frame.xoffLabel -text XOff -width 5
    scale $frame.xoffScale -from 0.0 -to 1.0 -length $scaleWidth -resolution 0.02 -orient horizontal \
    -variable ${varnm}(xoff) -showvalue false -command "DrawSlice x xoff"
    add_tip $frame.xoffScale specAttr.xoffScale "Use this scale bar to set  the value of the offset used when drawing horizontal vectors.  The value can range from 0 to 1 and specifies the fractional position at which a data point with intensity zero will be drawn."
    
    entry $frame.xoffEntry -width $entryWidth -relief sunken -textvariable ${varnm}(xoff)
    bind $frame.xoffEntry <Return> "DrawSlice x xoff"
    add_tip $frame.xoffEntry specAttr.xoffEntry "Enter the value of the offset used when drawing horizontal vectors.  The value can range from 0 to 1 and specifies the fractional position at which a data point with intensity zero will be drawn."
    
    pack $frame.xoffLabel $frame.xoffScale $frame.xoffEntry -side left
    
    
    set frame $right.yoffFrame
    label $frame.yoffLabel -text YOff -width 5
    scale $frame.yoffScale -from 0.0 -to 1.0 -length $scaleWidth -resolution 0.02 -orient horizontal \
    -variable ${varnm}(yoff) -showvalue false -command "DrawSlice x yoff"
    add_tip $frame.yoffScale specAttr.yoffScale "Use this scale bar to set  the value of the offset used when drawing vertical vectors.  The value can range from 0 to 1 and specifies the fractional position at which a data point with intensity zero will be drawn."
    entry $frame.yoffEntry -width $entryWidth -relief sunken -textvariable ${varnm}(yoff)
    bind $frame.yoffEntry <Return> "DrawSlice y yoff"
    add_tip $frame.yoffEntry specAttr.yoffEntry "Enter the value of the offset used when drawing horizontal vectors.  The value can range from 0 to 1 and specifies the fractional position at which a data point with intensity zero will be drawn."
    pack $frame.yoffLabel $frame.yoffScale $frame.yoffEntry -side left
    
    set panvar(lvl) 11
    set panvar(clm) 1.2
    set panvar(scale) 10.0
    
    frame $nm.limFrame
    pack $nm.limFrame -side top -fill both
    frame $nm.limFrame.txt
    pack $nm.limFrame.txt -side left -fill both
    foreach row "x y z z2" {
        set frame $nm.limFrame.txt.fr${row}
        frame $frame
        pack $frame -side top -fill x -padx 2 -pady 2
        menubutton $frame.x -text [string toupper $row] -underline 0 -menu $frame.x.menu -width 3 -relief raised
        add_tip $frame.x specAttr.menu.$row "Click on this menu button to get a menu of preset plot limits for the $row axis"
        menu $frame.x.menu -tearoff 0
        $frame.x.menu add command   -label Alpha -command "nv_win $row 4.0  7.0"
        $frame.x.menu add command   -label Amide -command "nv_win $row 6.0 10.0"
        $frame.x.menu add command   -label Full -command "nv_win pt $row -100000 100000"
        $frame.x.menu add command   -label First -command "nv_win pt $row 1 1"
        $frame.x.menu add command   -label Last -command "nv_win pt $row  100000 100000"
        entry $frame.x0 -width 8 -relief sunken -textvariable ${varnm}($row,min)
        bind $frame.x0 <Return> "set ${varnm}($row,max) \$${varnm}($row,min)"
        add_tip $frame.x0 specAttr.plotlim0.$row "Enter a plot limit for the $row axis.  Hitting the Enter key \
        will set this same value into the second plot limit entry for the $row axis."
        
        entry $frame.x1 -width 8 -relief sunken -textvariable ${varnm}($row,max)
        add_tip $frame.x1 specAttr.plotlim1.$row "Enter the second plot limit for the $row axis.  For the z and z2 dimensions\
        all planes between (inclusive) the first and second plot limits are drawn."
        
        entry $frame.lab -width 8 -relief sunken -textvariable ${varnm}($row,lab)
        add_tip $frame.lab specAttr.plotlab.$row "Enter a label for the $row axis.  Normally this is extracted  automatically when a dataset is assigned to\
        the spectral window.  It can be changed manually to disable cursor correlation"
        
        
        menubutton $frame.dim -text 1 -underline 0 -menu $frame.dim.menu -width 3 -relief raised -textvariable ${varnm}(${row}dim)
        add_tip $frame.dim specAttr.dim$row "Use this menu to select which dataset dimension is displayed on  the $row axis of the plot."
        pack $frame.x -side left
        pack $frame.x0 $frame.x1 $frame.lab -side left
        pack $frame.dim -side left -padx 2
        menu $frame.dim.menu -tearoff 0
        $frame.dim.menu add radiobutton -label 1 -variable ${varnm}(${row}dim) -value 1 -command "nv_win ${row}dim 1"
        $frame.dim.menu add radiobutton -label 2 -variable ${varnm}(${row}dim) -value 2 -command "nv_win ${row}dim 2"
        $frame.dim.menu add radiobutton -label 3 -variable ${varnm}(${row}dim) -value 3 -command "nv_win ${row}dim 3"
        $frame.dim.menu add radiobutton -label 4 -variable ${varnm}(${row}dim) -value 4 -command "nv_win ${row}dim 4"
    }
    frame $nm.limFrame.grph
    pack $nm.limFrame.grph -side left -fill both -expand yes
    set frame $nm.limFrame.grph
    canvas $frame.c -width 100 -height 100
    add_tip $frame.c specAttr.graph "The blue box shows the currently active plot region.  You may drag\
    it with the the mouse with button 1 down, or resize it with the mouse with button 2 down."
    pack $frame.c
    set limits "2 2 99 99"
    eval $frame.c create rectangle $limits -tags Limits
    $frame.c create line 2 99 99 2
    $frame.c create line 2 2 99 99
    $frame.c create line 50 2 50 99
    $frame.c create line 2 50 99 50
    $frame.c create rectangle 1c 1c 2c 2c -tags Rect -fill blue -outline blue
    $frame.c lower Rect
    
    bind $frame.c <$Nv_Priv(ButtonPress1)> "Nv_SetAdjustMode $frame.c Rect %x %y Move"
    bind $frame.c <$Nv_Priv(ButtonPress2)> "Nv_SetAdjustMode $frame.c Rect %x %y Resize"
    bind $frame.c <$Nv_Priv(B1Motion)> "
    Nv_AdjustCanvItem $frame.c Rect Limits %x %y
    NvSetPPM $frame.c Rect Limits $varnm
    "
    bind $frame.c <$Nv_Priv(B2Motion)> "
    Nv_AdjustCanvItem $frame.c Rect Limits %x %y
    NvSetPPM $frame.c Rect Limits $varnm
    "
    bind $frame.c <$Nv_Priv(ButtonRelease1)> "
    Nv_AdjustCanvItem $frame.c Rect Limits %x %y
    NvSetPPM $frame.c Rect Limits $varnm
    NvSpectrumDraw
    "
    bind $frame.c <$Nv_Priv(ButtonRelease2)> "
    Nv_AdjustCanvItem $frame.c Rect Limits %x %y
    NvSetPPM $frame.c Rect Limits $varnm
    NvSpectrumDraw
    "
    
    wm withdraw $nm
}

proc NvChangeLvl {direc varnm} {
    upvar #0 $varnm panvar
    if {$direc=="up"} {
        set panvar(lvl) [expr $panvar(lvl)*$panvar(clm)]
    } else {
        set panvar(lvl) [expr $panvar(lvl)/$panvar(clm)]
        if {$panvar(lvl)<0.0} {
            set panvar(lvl) 0.0
        }
    }
}

proc NvChangeScale {mul window} {
    set to [$window cget -to]
    set from [$window cget -from]
    set resolution [$window cget -resolution]
    $window config -to [expr $to*$mul]
    $window config -from [expr $from*$mul]
    $window config -resolution [expr $resolution*$mul]
}
proc errmsg {text} {
    puts stdout $text
}

proc NvShowSpecAttr args {
    global Nv_Priv
    
    if {[llength $args]>0} {
        set nm [lindex $args 0]
    } else {
        set nm $Nv_Priv(specAttr)
    }
    
    wm deiconify $nm
    raise $nm
    NvUpdateSpecAttr $nm
    
}

proc NvUpdateSpecAttr args {
    global Nv_Priv
    
    if {[llength $args]>0} {
        set nm [lindex $args 0]
    } else {
        set nm $Nv_Priv(specAttr)
    }
    
    set varnm [NvVarnm $nm]
    upvar #0 $varnm panvar
    wm title $nm "$nm: [nv_win nam]"
    if {[catch "nv_win dataset"]} {
        wm title $nm "$nm: [nv_win nam]"
        return
    } else {
        wm title $nm "$nm: [nv_win nam]: [nv_win dataset]"
    }
    foreach var "lvl scale xoff yoff nlvl clm  poscolor negcolor dim mode aclear" {
        set panvar($var) [nv_win $var]
    }
    
    set axMode ppm
    set i 0
    
    foreach var "x y z z2" {
        if {$i<[nv_win ndim]} {
            set minPPM [lindex [nv_win ppm ${var}] 0]
            set maxPPM [lindex [nv_win ppm ${var}] 1]
            if {$axMode=="pt"} {
                set minPt [expr round([lindex [nv_win $axMode ${var}] 0])]
                set maxPt [expr round([lindex [nv_win $axMode ${var}] 1])]
                set panvar($var,min) ${minPt}p
                set panvar($var,max) ${maxPt}p
            } else {
                set panvar($var,min) $minPPM
                set panvar($var,max) $maxPPM
            }
            
            set panvar(${var}dim) [nv_win ${var}dim]
            set panvar($var,lab) [nv_win lab $var]
            
            set minVal [lindex [nv_win ppmMax $var] 0]
            set maxVal [lindex [nv_win ppmMax $var] 1]
            
            if {$maxVal==$minVal} {
                set f${var}1 1.0
                set f${var}2 1.0
            } else {
                set val $minPPM
                set f${var}1 [expr ($val-$minVal)/($maxVal-$minVal)]
                
                set val $maxPPM
                set f${var}2 [expr ($val-$minVal)/($maxVal-$minVal)]
            }
            
        } else {
            set panvar($var,min) ""
            set panvar($var,max) ""
            set panvar(${var}dim) $i
            set panvar($var,lab) ""
        }
        incr i
    }
    set fx1 [expr 1.0-$fx1]
    set fx2 [expr 1.0-$fx2]
    Nv_SetCanvItem $nm.limFrame.grph.c Rect Limits $fx1 $fy1 $fx2 $fy2
    return
}

proc NvGetSpecAttr args {
    global Nv_Priv
    if {[llength $args]>0} {
        set nm [lindex $args 0]
    } else {
        set nm $Nv_Priv(specAttr)
    }
    
    set varnm [NvVarnm $nm]
    upvar #0 $varnm panvar
    nv_win defer on
    foreach var "lvl scale xoff yoff nlvl clm  poscolor negcolor dim mode aclear" {
        nv_win $var $panvar($var)
    }
    set i 0
    foreach var "x y z z2" {
        if {$i<[nv_win ndim]}  {
            nv_win ${var} $panvar($var,min) $panvar($var,max)
            nv_win ${var}dim $panvar(${var}dim)
            nv_win lab ${var} $panvar($var,lab)
        }
        incr i
        
    }
    nv_win defer off
    return
}

proc NvSetAttrPar {varnm var value } {
    return
    nv_win defer on
    nv_win ${var} $value
    nv_win defer off
}

proc DrawSlice  {ax var value } {
    nv_win defer on
    nv_win ${var} $value
    if {[nv_win slice $ax]} {
        eval nv_win cross1 [nv_win cross1]
    }
    nv_win defer off
}

proc NvShowDatasetList args {
    global Nv_Priv NvSpectrum
    
    if {[llength $args]>0} {
        set nm [lindex $args 0]
    } else {
        set nm $Nv_Priv(datasetList)
    }
    set varnm [NvVarnm $nm]
    upvar #0 $varnm SpecData
    set SpecData(window) [nv_win active]
    
    $nm.filesAssigned delete 0 end
    set SpecData(selectAssigned) ""
    set SpecData(selectOpened) ""
    
    set datasets {}
    if {[catch "nv_win dataset" dataset]} {
        if {![info exists NvSpectrum($SpecData(window),datasets)]} {
            set NvSpectrum($SpecData(window),datasets) {}
        }
    } else {
        lappend NvSpectrum($SpecData(window),datasets) $dataset
    }
    set validDatasets [nv_dataset open]
    
    foreach dataset $NvSpectrum($SpecData(window),datasets) {
        if {[lsearch $validDatasets $dataset]>=0} {
            if {[lsearch $datasets $dataset]<0} {
                lappend datasets $dataset
            }
        }
    }
    
    set NvSpectrum($SpecData(window),datasets) $datasets
    
    
    foreach dataset $NvSpectrum($SpecData(window),datasets) {
        $nm.filesAssigned insert end $dataset
    }
    
    wm title $nm DatasetMngr:[nv_win active]
    wm deiconify $nm
    raise $nm
}


proc NvMkDatasetList {nm} {
    toplevel $nm
    set varnm [NvVarnm $nm]
    
    wm withdraw $nm
    
    set frame $nm.top
    frame $frame
    pack $frame -side top -fill both -expand yes
    
    set frame $nm.top.lfr
    frame $frame
    pack $frame -side left -fill both -expand yes
    
    set frame $nm.top.lfr.listfr
    frame $frame
    pack $frame -side top -fill both -expand yes
    
    listbox $nm.filesOpen -height 6 -yscrollcommand "$frame.lscroll set"
    bind $nm.filesOpen <ButtonRelease-1> "set ${varnm}(selectOpened) \[selection get\]"
    bind $nm.filesOpen <Double-Button-1> "NvSpecDataAdd $nm"
    add_tip $nm.filesOpen datasetList.filesOpen "This is a list of currently opened datasets."
    scrollbar $frame.lscroll -command "$nm.filesOpen yview"
    
    pack $nm.filesOpen -in $nm.top.lfr.listfr -side left -fill both -expand yes
    pack $frame.lscroll -side left -fill y
    
    set label $nm.top.lfr.select
    label $label -textvariable ${varnm}(selectOpened) -relief sunken
    pack $label -side bottom -fill x
    
    
    set frame $nm.top.rfr
    frame $frame
    pack $frame -side left -fill both -expand yes
    
    set frame $nm.top.rfr.listfr
    frame $frame
    pack $frame -side top -fill both -expand yes
    
    listbox $nm.filesAssigned -height 6 -yscrollcommand "$frame.rscroll set"
    bind $nm.filesAssigned <ButtonRelease-1> "set ${varnm}(selectAssigned) \[selection get\]"
    add_tip $nm.filesAssigned datasetList.filesAssigned "This is a list of datasets assigned to this window."
    scrollbar $frame.rscroll -command "$nm.filesAssigned yview"
    
    pack $nm.filesAssigned -in $nm.top.rfr.listfr -side left -fill both -expand yes
    pack $frame.rscroll -side left -fill y
    
    set label $nm.top.rfr.select
    label $label -textvariable ${varnm}(selectAssigned) -relief sunken
    pack $label -side bottom -fill x
    
    
    set frame $nm.bbar
    frame $frame
    pack $frame -fill x -side bottom
    button $frame.add -text Add -command "NvSpecDataAdd $nm"
    add_tip $frame.add datasetList.add "Add the selected dataset to the list of datasets displayed in this spectrum.\n\n Note: At present all datasets\
    assigned to the same spectrum must have the same size and referencing."
    button $frame.replace -text Replace -command "NvSpecDataReplace $nm"
    add_tip $frame.replace datasetList.replace "Replace all datasets currently assigned to this spectrum with the selected dataset."
    button $frame.remove -text Remove -command "NvSpecDataRemove $nm"
    add_tip $frame.remove datasetList.remove "Remove the selected dataset from the list of datasets assigned to this spectrum."
    button $frame.close -text Close -command "wm withdraw $nm"
    add_tip $frame.close datasetList.close "Close this panel."
    pack $frame.add $frame.replace $frame.remove $frame.close -side left -expand yes
}

proc NvSpecDataAdd {tl} {
    global NvSpectrum
    set varnm [NvVarnm $tl]
    upvar #0 $varnm SpecData
    for {set i 0} {$i<[$tl.filesAssigned size]} {incr i} {
        set entry [$tl.filesAssigned get $i]
        if {[string compare $entry $SpecData(selectOpened)]==0} {
            return
        }
    }
    
    $tl.filesAssigned insert end $SpecData(selectOpened)
    set firstDataset [$tl.filesAssigned get 0]
    $SpecData(window) dataset $firstDataset
    
    set NvSpectrum($SpecData(window),datasets) ""
    
    for {set i 0} {$i<[$tl.filesAssigned size]} {incr i} {
        set entry [$tl.filesAssigned get $i]
        lappend NvSpectrum($SpecData(window),datasets) $entry
    }
}
proc NvSpecDataReplace {tl} {
    global NvSpectrum
    set varnm [NvVarnm $tl]
    upvar #0 $varnm SpecData
    
    $tl.filesAssigned delete 0 end
    
    $tl.filesAssigned insert end $SpecData(selectOpened)
    $SpecData(window) dataset $SpecData(selectOpened)
    
    set NvSpectrum($SpecData(window),datasets) $SpecData(selectOpened)
}

proc NvSpecDataRemove {tl} {
    global NvSpectrum
    set varnm [NvVarnm $tl]
    upvar #0 $varnm SpecData
    for {set i 0} {$i<[$tl.filesAssigned size]} {incr i} {
        set entry [$tl.filesAssigned get $i]
        if {$entry == $SpecData(selectAssigned)} {
            $tl.filesAssigned delete $i
            break
        }
    }
    update
    set NvSpectrum($SpecData(window),datasets) ""
    for {set i 0} {$i<[$tl.filesAssigned size]} {incr i} {
        set entry [$tl.filesAssigned get $i]
        lappend NvSpectrum($SpecData(window),datasets) $entry
    }
}




proc NvFillDatasetOpen {nm } {
    $nm.filesOpen delete 0 end
    foreach i [nv_dataset open] {
        $nm.filesOpen insert end $i
    }
}

proc NvSetPPM {canvas item borderItem varnm} {
    upvar #0 $varnm panvar
    set flimits [Nv_GetViewFrac $canvas $item $borderItem]
    
    set fx1 [lindex $flimits 0]
    set fy1 [lindex $flimits 1]
    set fx2 [lindex $flimits 2]
    set fy2 [lindex $flimits 3]
    
    set fx1 [expr 1.0-$fx1]
    set fx2 [expr 1.0-$fx2]
    
    set minVal [lindex [nv_win ppmMax x] 0]
    set maxVal [lindex [nv_win ppmMax x] 1]
    
    set x1 [expr $fx1*($maxVal-$minVal)+$minVal]
    set x2 [expr $fx2*($maxVal-$minVal)+$minVal]
    
    set minVal [lindex [nv_win ppmMax y] 0]
    set maxVal [lindex [nv_win ppmMax y] 1]
    
    set y1 [expr $fy1*($maxVal-$minVal)+$minVal]
    set y2 [expr $fy2*($maxVal-$minVal)+$minVal]
    
    set panvar(x,min) $x1
    set panvar(x,max) $x2
    set panvar(y,min) $y1
    set panvar(y,max) $y2
    
}

